{==============================================================================}
{ DSPLab                                                                       }
{ Copyright 2003 Tarasov Eugen                                                 }
{ How to build frequency meter using DSPLab                                    }
{ Please see FreqMeter.pas - implementation of simple frequency meter          }
{                                                                              }
{ Last modified January, 2003                                                  }
{                                                                              }
{ e-mail: dsplab@teworks.com                                                   }
{ http://www.teworks.com                                                       }
{==============================================================================}

unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, ComCtrls, Buttons, TeEngine, Series, TeeProcs, Chart,
  Menus, Math, FreqMeter;

type
  TfmMain = class(TForm)
    Panel2: TPanel;
    StatusBar1: TStatusBar;
    Timer1: TTimer;
    ScrollBox1: TScrollBox;
    GeneratorBox: TGroupBox;
    lFreq1: TLabel;
    lAmp1: TLabel;
    tbChannelFreq1: TTrackBar;
    rbSin1: TRadioButton;
    rbSquare1: TRadioButton;
    rbTriangle1: TRadioButton;
    edtChannelFreq1: TEdit;
    tbChannelAmp1: TTrackBar;
    edtChannelAmp1: TEdit;
    Chart1: TChart;
    Series1: TFastLineSeries;
    GroupBox1: TGroupBox;
    FreqValue: TEdit;
    FreqTimer: TTimer;
    procedure rbSin1Click(Sender: TObject);
    procedure tbChannelFreq1Change(Sender: TObject);
    procedure tbChannelAmp1Change(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure FreqTimerTimer(Sender: TObject);
  private
    SignalForm: Integer;
    SignalAmp, SignalFreq: Integer;
    FrequencyMeter: TFrequencyMeter;
  public
    { Public declarations }
  end;

var
  fmMain: TfmMain;

implementation

{$R *.DFM}

const
  cGeneratorSampleRate = 1000; // 1000 Hz


procedure TfmMain.FormCreate(Sender: TObject);
begin
  Application.Title:= 'DSPLab - ' + Caption;
  Chart1.BottomAxis.Maximum:= 0;

  // Create the frequency meter
  FrequencyMeter:= TFrequencyMeter.Create(Self);
  FrequencyMeter.SampleRate:= cGeneratorSampleRate;

  // Setup Generator
  tbChannelAmp1Change(tbChannelAmp1);
  tbChannelFreq1Change(tbChannelFreq1);

  // Start generator
  Timer1.Enabled:= true;
end;


procedure TfmMain.rbSin1Click(Sender: TObject);
begin
  SignalForm:= (Sender as TComponent).Tag;
end;


procedure TfmMain.tbChannelFreq1Change(Sender: TObject);
begin
  SignalFreq:= tbChannelFreq1.Position;
  edtChannelFreq1.Text:= IntToStr(SignalFreq);
end;


procedure TfmMain.tbChannelAmp1Change(Sender: TObject);
begin
  SignalAmp:= tbChannelAmp1.Position;
  edtChannelAmp1.Text:= IntToStr(SignalAmp);
end;


// Signal generator
procedure TfmMain.Timer1Timer(Sender: TObject);
const
  cMsInDay = 24 * 3600000;
  cScaleRange = 1 / cMsInDay;
var
  I: Integer;
  T: Double;
  V: Single;

  function Sine: Single;
  begin
    Result:= SignalAmp * Sin(2 * PI * SignalFreq * T);
  end;

  function Triangle: Single;
  begin
    Result:= Frac(SignalFreq * T) * 4;
    if Result <= 2 then Result:= -SignalAmp * (1 - Result)
    else Result:= SignalAmp * (3 - Result);
  end;

  function Rectangle: Single;
  begin
    Result:= Frac(SignalFreq * T) - 0.5;
    if Result < 0.0 then Result:= -SignalAmp else Result:= SignalAmp;
  end;
begin
  for I:= 1 to 5 do
  begin
    T:= Timer1.Tag / cGeneratorSampleRate;

    case SignalForm of
      0: V:= Sine;
      1: V:= Rectangle;
      2: V:= Triangle;
    else V:= 0;
    end;

    T:= T / cMsInDay;
    if T >= Chart1.BottomAxis.Maximum then
    begin
      Chart1.BottomAxis.SetMinMax(
        Chart1.BottomAxis.Maximum, Chart1.BottomAxis.Maximum + cScaleRange);
    end;

    Timer1.Tag:= Timer1.Tag + 1;

    // Display signal
    Series1.AddXY(T, V, '', clRed);

    //Calculate Frequency
    FrequencyMeter.Put(V);
  end;
end;


procedure TfmMain.FreqTimerTimer(Sender: TObject);
var S: String;
begin
  // Display Calculated Frequency
  Str(FrequencyMeter.Frequency:5:1, S);
  FreqValue.Text:= S;
end;

end.
